@@ -1,22 +1,19 @@
 import hashlib
+import uuid
 from abc import ABC
 from datetime import datetime
 from typing import Callable, List, Union
 
-from unstructured.documents.elements import Element, NoID, Text
+from unstructured.documents.elements import UUID, Element, NoID, Text
 
 
 class NoDatestamp(ABC):
     """Class to indicate that an element do not have a datetime stamp."""
 
-    pass
-
 
 class EmailElement(Element):
     """An email element is a section of the email."""
 
-    pass
-
 
 class Name(EmailElement):
     """Base element for capturing free text from within document."""
@@ -28,7 +25,7 @@ class Name(EmailElement):
         name: str,
         text: str,
         datestamp: Union[datetime, NoDatestamp] = NoDatestamp(),
-        element_id: Union[str, NoID] = NoID(),
+        element_id: Union[str, uuid.UUID, NoID, UUID] = NoID(),
     ):
         self.name: str = name
         self.text: str = text
@@ -37,6 +34,9 @@ class Name(EmailElement):
             # NOTE(robinson) - Cut the SHA256 hex in half to get the first 128 bits
             element_id = hashlib.sha256(text.encode()).hexdigest()[:32]
 
+        elif isinstance(element_id, UUID):
+            element_id = uuid.uuid4()
+
         super().__init__(element_id=element_id)
 
         if isinstance(datestamp, datetime):
@@ -80,32 +80,24 @@ class BodyText(List[Text]):
 
     category = "BodyText"
 
-    pass
-
 
 class Recipient(Name):
     """A text element for capturing the recipient information of an email"""
 
     category = "Recipient"
 
-    pass
-
 
 class Sender(Name):
     """A text element for capturing the sender information of an email"""
 
     category = "Sender"
 
-    pass
-
 
 class Subject(Text, EmailElement):
     """A text element for capturing the subject information of an email"""
 
     category = "Subject"
 
-    pass
-
 
 class MetaData(Name):
     """A text element for capturing header meta data of an email
@@ -113,21 +105,15 @@ class MetaData(Name):
 
     category = "MetaData"
 
-    pass
-
 
 class ReceivedInfo(Name):
     """A text element for capturing header information of an email (e.g. IP addresses, etc)."""
 
     category = "ReceivedInfo"
 
-    pass
-
 
 class Attachment(Name):
     """A text element for capturing the attachment name in an email (e.g. documents,
     images, etc)."""
 
     category = "Attachment"
-
-    pass
