@@ -1,7 +1,9 @@
+# pyright: reportPrivateUsage=false
+
 from __future__ import annotations
 
 import sys
-from typing import List, Optional, Sequence, Tuple
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, cast
 
 if sys.version_info < (3, 8):
     from typing_extensions import Final
@@ -15,10 +17,12 @@ from unstructured.documents.base import Page
 from unstructured.documents.elements import (
     Address,
     Element,
+    ElementMetadata,
     EmailAddress,
     Link,
     ListItem,
     NarrativeText,
+    Table,
     Text,
     Title,
 )
@@ -31,28 +35,35 @@ from unstructured.partition.text_type import (
     is_possible_title,
     is_us_city_state_zip,
 )
+from unstructured.utils import htmlify_matrix_of_cell_texts
 
 TEXT_TAGS: Final[List[str]] = ["p", "a", "td", "span", "font"]
 LIST_ITEM_TAGS: Final[List[str]] = ["li", "dd"]
+LIST_TAGS: Final[List[str]] = ["ul", "ol", "dl"]
 HEADING_TAGS: Final[List[str]] = ["h1", "h2", "h3", "h4", "h5", "h6"]
 TABLE_TAGS: Final[List[str]] = ["table", "tbody", "td", "tr"]
+TEXTBREAK_TAGS: Final[List[str]] = ["br"]
 PAGEBREAK_TAGS: Final[List[str]] = ["hr"]
+EMPTY_TAGS: Final[List[str]] = PAGEBREAK_TAGS + TEXTBREAK_TAGS
 HEADER_OR_FOOTER_TAGS: Final[List[str]] = ["header", "footer"]
-EMPTY_TAGS: Final[List[str]] = ["br", "hr"]
 SECTION_TAGS: Final[List[str]] = ["div", "pre"]
 
 
+# -- HTML-specific document-elements and methods -------------------------------------------------
+
+
 class TagsMixin:
     """Mixin that allows a class to retain tag information."""
 
     def __init__(
         self,
-        *args,
+        *args: Any,
         tag: Optional[str] = None,
         ancestortags: Sequence[str] = (),
         links: Sequence[Link] = [],
-        emphasized_texts: Sequence[dict] = [],
-        **kwargs,
+        emphasized_texts: Sequence[Dict[str, str]] = [],
+        text_as_html: Optional[str] = None,
+        **kwargs: Any,
     ):
         if tag is None:
             raise TypeError("tag argument must be passed and not None")
@@ -61,43 +72,57 @@ class TagsMixin:
         self.ancestortags = ancestortags
         self.links = links
         self.emphasized_texts = emphasized_texts
+        self.text_as_html = text_as_html
         super().__init__(*args, **kwargs)
 
 
 class HTMLText(TagsMixin, Text):
     """Text with tag information."""
 
-    pass
-
 
 class HTMLAddress(TagsMixin, Address):
     """Address with tag information."""
 
-    pass
-
 
 class HTMLEmailAddress(TagsMixin, EmailAddress):
     """EmailAddress with tag information"""
 
-    pass
-
 
 class HTMLTitle(TagsMixin, Title):
     """Title with tag information."""
 
-    pass
-
 
 class HTMLNarrativeText(TagsMixin, NarrativeText):
     """NarrativeText with tag information."""
 
-    pass
-
 
 class HTMLListItem(TagsMixin, ListItem):
     """NarrativeText with tag information."""
 
-    pass
+
+class HTMLTable(TagsMixin, Table):
+    """NarrativeText with tag information"""
+
+
+def has_table_ancestor(element: TagsMixin) -> bool:
+    """Checks to see if an element has ancestors that are table elements. If so, we consider
+    it to be a table element rather than a section of narrative text."""
+    return any(ancestor in TABLE_TAGS for ancestor in element.ancestortags)
+
+
+def in_header_or_footer(element: TagsMixin) -> bool:
+    """Checks to see if an element is contained within a header or a footer tag."""
+    if any(ancestor in HEADER_OR_FOOTER_TAGS for ancestor in element.ancestortags):
+        return True
+    return False
+
+
+def is_table(element: TagsMixin) -> bool:
+    """Checks to see if an element is a table"""
+    return element.tag in TABLE_TAGS
+
+
+# -- HTML element-tree processing ----------------------------------------------------------------
 
 
 class HTMLDocument(XMLDocument):
@@ -113,22 +138,26 @@ class HTMLDocument(XMLDocument):
         self.assembled_articles = assemble_articles
         super().__init__(stylesheet=stylesheet, parser=parser)
 
-    def _read(self) -> List[Page]:
-        """Reads and structures and HTML document. If present, looks for article tags.
-        if there are multiple article sections present, a page break is inserted between them.
+    def _parse_pages_from_element_tree(self) -> List[Page]:
+        """Parse HTML elements into pages.
+
+        A *page* is a subsequence of the document-elements parsed from the HTML document
+        corresponding to a distinct topic. At present pagination is determined by `<article>`
+        elements that surround something like a blog-post. Each article becomes its own page. If no
+        article tags are present in the HTML the entire HTML document is a single page.
         """
         if self._pages:
             return self._pages
         logger.info("Reading document ...")
         pages: List[Page] = []
-        etree.strip_elements(self.document_tree, ["script"])
+        etree.strip_elements(self.document_tree, ["script", "style"], with_tail=False)
         root = _find_main(self.document_tree)
 
         articles = _find_articles(root, assemble_articles=self.assembled_articles)
         page_number = 0
         page = Page(number=page_number)
         for article in articles:
-            descendanttag_elems: Tuple[etree.Element, ...] = ()
+            descendanttag_elems: Tuple[etree._Element, ...] = ()
             for tag_elem in article.iter():
                 if tag_elem in descendanttag_elems:
                     # Prevent repeating something that's been flagged as text as we chase it
@@ -136,15 +165,32 @@ class HTMLDocument(XMLDocument):
                     continue
 
                 if _is_text_tag(tag_elem):
-                    element = _parse_tag(tag_elem)
-                    if element is not None:
-                        page.elements.append(element)
-                        descendanttag_elems = tuple(tag_elem.iterdescendants())
+                    if _has_break_tags(tag_elem):
+                        flattened_elems = _unfurl_break_tags(tag_elem)
+                        for _tag_elem in flattened_elems:
+                            element = _parse_tag(_tag_elem)
+                            if element is not None:
+                                page.elements.append(element)
+
+                    else:
+                        element = _parse_tag(tag_elem)
+                        if element is not None:
+                            page.elements.append(element)
+                    descendanttag_elems = tuple(tag_elem.iterdescendants())
 
                 elif _is_container_with_text(tag_elem):
                     links = _get_links_from_tag(tag_elem)
                     emphasized_texts = _get_emphasized_texts_from_tag(tag_elem)
-                    element = _text_to_element(tag_elem.text, "div", (), links, emphasized_texts)
+                    # -- having text is guaranteed by `_is_container_with_text()` --
+                    assert tag_elem.text is not None
+                    element = _text_to_element(
+                        tag_elem.text,
+                        "div",
+                        (),
+                        depth=0,
+                        links=links,
+                        emphasized_texts=emphasized_texts,
+                    )
                     if element is not None:
                         page.elements.append(element)
 
@@ -157,7 +203,17 @@ class HTMLDocument(XMLDocument):
                     element, next_element = _process_list_item(tag_elem)
                     if element is not None:
                         page.elements.append(element)
-                        descendanttag_elems = _get_bullet_descendants(tag_elem, next_element)
+                        descendanttag_elems = _get_bullet_descendants(
+                            tag_elem,
+                            next_element,
+                        )
+
+                elif tag_elem.tag in TABLE_TAGS:
+                    element = _parse_HTMLTable_from_table_elem(tag_elem)
+                    if element is not None:
+                        page.elements.append(element)
+                    if element or tag_elem.tag == "table":
+                        descendanttag_elems = tuple(tag_elem.iterdescendants())
 
                 elif tag_elem.tag in PAGEBREAK_TAGS and len(page.elements) > 0:
                     pages.append(page)
@@ -173,30 +229,31 @@ class HTMLDocument(XMLDocument):
 
     def doc_after_cleaners(
         self,
-        skip_headers_and_footers=False,
-        skip_table_text=False,
-        inplace=False,
+        skip_headers_and_footers: bool = False,
+        skip_table: bool = False,
+        inplace: bool = False,
     ) -> HTMLDocument:
-        """Filters the elements and returns a new instance of the class based on the criteria
-        specified. Note that the number of pages can change in the case that all elements on a
-        page are filtered out.
+        """Filters elements returning new instance based on the criteria specified.
+
+        Note that the number of pages can change in the case that all elements on a page are
+        filtered out.
+
         Parameters
         ----------
-        skip_table_text:
-            If True, skips text that is contained within a table element
+        skip_table:
+            If True, skips table element
         skip_headers_and_footers:
             If True, ignores any content that is within <header> or <footer> tags
         inplace:
             If True, document is modified in place and returned.
         """
-
-        excluders = []
+        excluders: List[Callable[[TagsMixin], bool]] = []
         if skip_headers_and_footers:
             excluders.append(in_header_or_footer)
-        if skip_table_text:
-            excluders.append(has_table_ancestor)
+        if skip_table:
+            excluders.append(is_table)
 
-        pages = []
+        pages: List[Page] = []
         page_number = 0
         new_page = Page(number=page_number)
         for page in self.pages:
@@ -227,30 +284,87 @@ class HTMLDocument(XMLDocument):
                 # NOTE(robinson) - Skipping for test coverage because this condition is impossible.
                 # Added type check because from_pages is a method on Document. Without the type
                 # check, mypy complains about returning Document instead of HTMLDocument
-                raise ValueError(f"Unexpected class: {self.__class__.__name__}")  # pragma: no cover
+                raise ValueError(
+                    f"Unexpected class: {self.__class__.__name__}",
+                )  # pragma: no cover
             return out
 
 
-def _get_links_from_tag(tag_elem: etree.Element) -> List[Link]:
+def _get_links_from_tag(tag_elem: etree._Element) -> List[Link]:
+    """Hyperlinks within and below `tag_elem`."""
     links: List[Link] = []
     href = tag_elem.get("href")
+    # TODO(klaijan) - add html href start_index
     if href:
-        links.append({"text": tag_elem.text, "url": href})
-
+        links.append({"text": tag_elem.text, "url": href, "start_index": -1})
     for tag in tag_elem.iterdescendants():
         href = tag.get("href")
         if href:
-            links.append({"text": tag.text, "url": href})
+            links.append({"text": tag.text, "url": href, "start_index": -1})
     return links
 
 
-def _get_emphasized_texts_from_tag(tag_elem: etree.Element) -> List[dict]:
-    """Get emphasized texts enclosed in <strong>, <em>, <span>, <b>, <i> tags
-    from a tag element in HTML"""
-    emphasized_texts = []
+def _is_bulleted_table(table_elem: etree._Element) -> bool:
+    """True when all text in `table_elem` is bulleted text.
+
+    A table-row containing no text is not considered, but at least one bulleted-text item must be
+    present. A table with no text in any row is not a bulleted table.
+    """
+    if table_elem.tag != "table":
+        return False
+
+    trs = table_elem.findall(".//tr")
+    tr_texts = [_construct_text(tr) for tr in trs]
+
+    # -- a table with no text is not a bulleted table --
+    if all(not text for text in tr_texts):
+        return False
+
+    # -- all non-empty rows must contain bulleted text --
+    if any(text and not is_bulleted_text(text) for text in tr_texts):
+        return False
+
+    return True
+
+
+def _parse_HTMLTable_from_table_elem(table_elem: etree._Element) -> Optional[Element]:
+    """Form `HTMLTable` element from `tbl_elem`."""
+    if table_elem.tag != "table":
+        return None
+
+    # -- NOTE that this algorithm handles a nested-table by parsing all of its text into the text
+    # -- for the _cell_ containing the table (and this is recursive, so a table nested within a
+    # -- cell within the table within the cell too.)
+
+    trs = cast(
+        List[etree._Element], table_elem.xpath("./tr | ./thead/tr | ./tbody/tr | ./tfoot/tr")
+    )
+
+    if not trs:
+        return None
+
+    table_data = [[str(text) for text in tr.itertext()] for tr in trs]
+    html_table = htmlify_matrix_of_cell_texts(table_data)
+    table_text = " ".join(" ".join(row) for row in table_data).strip()
+
+    if table_text == "":
+        return None
+
+    return HTMLTable(
+        text=table_text,
+        text_as_html=html_table,
+        tag=table_elem.tag,
+        ancestortags=tuple(el.tag for el in table_elem.iterancestors())[::-1],
+    )
+
+
+def _get_emphasized_texts_from_tag(tag_elem: etree._Element) -> List[Dict[str, str]]:
+    """Emphasized text within and below `tag_element`.
+
+    Emphasis is indicated by `<strong>`, `<em>`, `<span>`, `<b>`, `<i>` tags.
+    """
+    emphasized_texts: List[Dict[str, str]] = []
     tags_to_track = ["strong", "em", "span", "b", "i"]
-    if tag_elem is None:
-        return []
 
     if tag_elem.tag in tags_to_track:
         text = _construct_text(tag_elem, False)
@@ -266,16 +380,28 @@ def _get_emphasized_texts_from_tag(tag_elem: etree.Element) -> List[dict]:
 
 
 def _parse_tag(
-    tag_elem: etree.Element,
+    tag_elem: etree._Element,
 ) -> Optional[Element]:
-    """Converts an etree element to a Text element if there is applicable text in the element.
-    Ancestor tags are kept so they can be used for filtering or classification without
-    processing the document tree again. In the future we might want to keep descendants too,
-    but we don't have a use for them at the moment."""
+    """Parses `tag_elem` to a Text element if it contains qualifying text.
+
+    Ancestor tags are kept so they can be used for filtering or classification without processing
+    the document tree again. In the future we might want to keep descendants too, but we don't have
+    a use for them at the moment.
+    """
     ancestortags: Tuple[str, ...] = tuple(el.tag for el in tag_elem.iterancestors())[::-1]
     links = _get_links_from_tag(tag_elem)
     emphasized_texts = _get_emphasized_texts_from_tag(tag_elem)
 
+    depth = (
+        # TODO(newel): Check the surrounding divs to see if should be root level
+        # -- zero index the depth --
+        int(tag_elem.tag[1]) - 1
+        if tag_elem.tag in HEADING_TAGS
+        else len([el for el in tag_elem.iterancestors() if el.tag in LIST_TAGS + LIST_ITEM_TAGS])
+        if tag_elem.tag in LIST_TAGS + LIST_ITEM_TAGS
+        else 0
+    )
+
     if tag_elem.tag == "script":
         return None
     text = _construct_text(tag_elem)
@@ -287,6 +413,7 @@ def _parse_tag(
         ancestortags,
         links=links,
         emphasized_texts=emphasized_texts,
+        depth=depth,
     )
 
 
@@ -294,11 +421,11 @@ def _text_to_element(
     text: str,
     tag: str,
     ancestortags: Tuple[str, ...],
+    depth: int,
     links: List[Link] = [],
-    emphasized_texts: List[dict] = [],
+    emphasized_texts: List[Dict[str, str]] = [],
 ) -> Optional[Element]:
-    """Given the text of an element, the tag type and the ancestor tags, produces the appropriate
-    HTML element."""
+    """Produce a document-element of the appropriate sub-type for `text`."""
     if is_bulleted_text(text):
         if not clean_bullets(text):
             return None
@@ -308,6 +435,7 @@ def _text_to_element(
             ancestortags=ancestortags,
             links=links,
             emphasized_texts=emphasized_texts,
+            metadata=ElementMetadata(category_depth=depth),
         )
     elif is_us_city_state_zip(text):
         return HTMLAddress(
@@ -318,7 +446,12 @@ def _text_to_element(
             emphasized_texts=emphasized_texts,
         )
     elif is_email_address(text):
-        return HTMLEmailAddress(text=text, tag=tag, links=links, emphasized_texts=emphasized_texts)
+        return HTMLEmailAddress(
+            text=text,
+            tag=tag,
+            links=links,
+            emphasized_texts=emphasized_texts,
+        )
 
     if len(text) < 2:
         return None
@@ -330,13 +463,14 @@ def _text_to_element(
             links=links,
             emphasized_texts=emphasized_texts,
         )
-    elif is_possible_title(text):
+    elif is_heading_tag(tag) or is_possible_title(text):
         return HTMLTitle(
             text,
             tag=tag,
             ancestortags=ancestortags,
             links=links,
             emphasized_texts=emphasized_texts,
+            metadata=ElementMetadata(category_depth=depth),
         )
     else:
         return HTMLText(
@@ -348,8 +482,9 @@ def _text_to_element(
         )
 
 
-def _is_container_with_text(tag_elem: etree.Element) -> bool:
+def _is_container_with_text(tag_elem: etree._Element) -> bool:
     """Checks if a tag is a container that also happens to contain text.
+
     Example
     -------
     <div>Hi there,
@@ -357,7 +492,7 @@ def _is_container_with_text(tag_elem: etree.Element) -> bool:
         <div>Please read my message!</div>
     </div>
     """
-    if tag_elem.tag not in SECTION_TAGS or len(tag_elem) == 0:
+    if tag_elem.tag not in SECTION_TAGS + ["body"] or len(tag_elem) == 0:
         return False
 
     if tag_elem.text is None or tag_elem.text.strip() == "":
@@ -371,12 +506,14 @@ def is_narrative_tag(text: str, tag: str) -> bool:
     return tag not in HEADING_TAGS and is_possible_narrative_text(text)
 
 
-def _construct_text(tag_elem: etree.Element, include_tail_text: bool = True) -> str:
-    """Extracts text from a text tag element."""
-    text = ""
-    for item in tag_elem.itertext():
-        if item:
-            text += item
+def is_heading_tag(tag: str) -> bool:
+    """Uses tag information to infer whether text is a heading."""
+    return tag in HEADING_TAGS
+
+
+def _construct_text(tag_elem: etree._Element, include_tail_text: bool = True) -> str:
+    """Extract "clean"" text from `tag_elem`."""
+    text = "".join(str(t) for t in tag_elem.itertext() if t)
 
     if include_tail_text and tag_elem.tail:
         text = text + tag_elem.tail
@@ -385,8 +522,41 @@ def _construct_text(tag_elem: etree.Element, include_tail_text: bool = True) ->
     return text.strip()
 
 
-def _is_text_tag(tag_elem: etree.Element, max_predecessor_len: int = 5) -> bool:
-    """Deteremines if a tag potentially contains narrative text."""
+def _has_break_tags(tag_elem: etree._Element) -> bool:
+    """True when `tab_elem` contains a `<br>` descendant."""
+    return any(descendant.tag in TEXTBREAK_TAGS for descendant in tag_elem.iterdescendants())
+
+
+def _unfurl_break_tags(tag_elem: etree._Element) -> List[etree._Element]:
+    """Sequence of `tag_elem` and its children with `<br>` elements removed.
+
+    NOTE that these are "loose" `etree._Element` instances that are NOT linked to the original HTML
+    element-tree, so methods like `.getchildren()`, `.find()` etc. will happily produce empty
+    results.
+    """
+    unfurled: List[etree._Element] = []
+
+    if tag_elem.text:
+        _tag_elem = etree.Element(tag_elem.tag)
+        _tag_elem.text = tag_elem.text
+        unfurled.append(_tag_elem)
+
+    children = tag_elem.getchildren()
+    for child in children:
+        if not _has_break_tags(child):
+            unfurled.append(child)
+        else:
+            if child.text:
+                _tag_elem = etree.Element(child.tag)
+                _tag_elem.text = child.text
+                unfurled.append(_tag_elem)
+            unfurled.extend(_unfurl_break_tags(child))
+
+    return unfurled
+
+
+def _is_text_tag(tag_elem: etree._Element, max_predecessor_len: int = 5) -> bool:
+    """True when `tag_element` potentially contains narrative text."""
     # NOTE(robinson) - Only consider elements with limited depth. Otherwise,
     # it could be the text representation of a giant div
     # Exclude empty tags from tag_elem
@@ -394,13 +564,13 @@ def _is_text_tag(tag_elem: etree.Element, max_predecessor_len: int = 5) -> bool:
     if len(tag_elem) > max_predecessor_len + empty_elems_len:
         return False
 
-    if tag_elem.tag in TEXT_TAGS + HEADING_TAGS:
+    if tag_elem.tag in TEXT_TAGS + HEADING_TAGS + TEXTBREAK_TAGS:
         return True
 
     # NOTE(robinson) - This indicates that a div tag has no children. If that's the
     # case and the tag has text, its potential a text tag
     children = tag_elem.getchildren()
-    if tag_elem.tag in SECTION_TAGS and len(children) == 0:
+    if tag_elem.tag in SECTION_TAGS + ["body"] and len(children) == 0:
         return True
 
     if _has_adjacent_bulleted_spans(tag_elem, children):
@@ -410,22 +580,28 @@ def _is_text_tag(tag_elem: etree.Element, max_predecessor_len: int = 5) -> bool:
 
 
 def _process_list_item(
-    tag_elem: etree.Element,
+    tag_elem: etree._Element,
     max_predecessor_len: int = 5,
-) -> Tuple[Optional[Element], etree.Element]:
-    """If an etree element contains bulleted text, extracts the relevant bulleted text
-    and converts it to ListItem objects. Also returns the next html elements so that
-    we can skip processing if bullets are found in a div element."""
-    if tag_elem.tag in LIST_ITEM_TAGS:
+) -> Tuple[Optional[Element], Optional[etree._Element]]:
+    """Produces an `HTMLListItem` document element from `tag_elem`.
+
+    When `tag_elem` contains bulleted text, the relevant bulleted text is extracted. Also returns
+    the next html element so we can skip processing if bullets are found in a div element.
+    """
+    if tag_elem.tag in LIST_TAGS + LIST_ITEM_TAGS:
         text = _construct_text(tag_elem)
         links = _get_links_from_tag(tag_elem)
         emphasized_texts = _get_emphasized_texts_from_tag(tag_elem)
+        depth = len(
+            [el for el in tag_elem.iterancestors() if el.tag in LIST_TAGS + LIST_ITEM_TAGS],
+        )
         return (
             HTMLListItem(
                 text=text,
                 tag=tag_elem.tag,
                 links=links,
                 emphasized_texts=emphasized_texts,
+                metadata=ElementMetadata(category_depth=depth),
             ),
             tag_elem,
         )
@@ -438,7 +614,9 @@ def _process_list_item(
         next_text = _construct_text(next_element)
         # NOTE(robinson) - Only consider elements with limited depth. Otherwise,
         # it could be the text representation of a giant div
-        empty_elems_len = len([el for el in tag_elem.getchildren() if el.tag in EMPTY_TAGS])
+        empty_elems_len = len(
+            [el for el in tag_elem.getchildren() if el.tag in EMPTY_TAGS],
+        )
         if len(tag_elem) > max_predecessor_len + empty_elems_len:
             return None, None
         if next_text:
@@ -447,27 +625,28 @@ def _process_list_item(
     return None, None
 
 
-def _get_bullet_descendants(element, next_element) -> Tuple[etree.Element, ...]:
-    descendants = []
-    if element is not None and next_element is not None:
-        descendants += list(next_element.iterdescendants())
-    descendanttag_elems = tuple(descendants)
-    return descendanttag_elems
+def _get_bullet_descendants(
+    element: Optional[etree._Element], next_element: Optional[etree._Element]
+) -> Tuple[etree._Element, ...]:
+    """Helper for list-item processing.
+
+    Gathers the descendants of `next_element` so they can be marked visited.
+    """
+    return () if element is None or next_element is None else tuple(next_element.iterdescendants())
 
 
-def is_list_item_tag(tag_elem: etree.Element) -> bool:
-    """Checks to see if a tag contains bulleted text."""
-    if tag_elem.tag in LIST_ITEM_TAGS or (
+def is_list_item_tag(tag_elem: etree._Element) -> bool:
+    """True when `tag_elem` contains bulleted text."""
+    return tag_elem.tag in LIST_ITEM_TAGS or (
         tag_elem.tag in SECTION_TAGS and is_bulleted_text(_construct_text(tag_elem))
-    ):
-        return True
-    return False
+    )
 
 
-def _bulleted_text_from_table(table) -> List[Element]:
-    """Extracts bulletized narrative text from a table.
-    NOTE: if a table has mixed bullets and non-bullets, only bullets are extracted.
-    I.e., _read() will drop non-bullet narrative text in the table.
+def _bulleted_text_from_table(table: etree._Element) -> List[Element]:
+    """Extracts bulletized narrative text from the `<table>` element in `table`.
+
+    NOTE: if a table has mixed bullets and non-bullets, only bullets are extracted. I.e., _read()
+    will drop non-bullet narrative text in the table.
     """
     bulleted_text: List[Element] = []
     rows = table.findall(".//tr")
@@ -478,23 +657,12 @@ def _bulleted_text_from_table(table) -> List[Element]:
     return bulleted_text
 
 
-def _is_bulleted_table(tag_elem) -> bool:
-    """Checks to see if a table element contains bulleted text."""
-    if tag_elem.tag != "table":
-        return False
+def _has_adjacent_bulleted_spans(tag_elem: etree._Element, children: List[etree._Element]) -> bool:
+    """True when `tag_elem` is a <div> or <pre> containing two or more adjacent bulleted spans.
 
-    rows = tag_elem.findall(".//tr")
-    for row in rows:
-        text = _construct_text(row)
-        if text and not is_bulleted_text(text):
-            return False
-
-    return True
-
-
-def _has_adjacent_bulleted_spans(tag_elem: etree.Element, children: List[etree.Element]) -> bool:
-    """Checks to see if a div contains two or more adjacent spans beginning with a bullet. If
-    this is the case, it is treated as a single bulleted text element."""
+    A bulleted span is one beginning with a bullet. If there are two or more adjacent to each other
+    they are treated as a single bulleted text element.
+    """
     if tag_elem.tag in SECTION_TAGS:
         all_spans = all(child.tag == "span" for child in children)
         _is_bulleted = children[0].text is not None and is_bulleted_text(children[0].text)
@@ -503,31 +671,20 @@ def _has_adjacent_bulleted_spans(tag_elem: etree.Element, children: List[etree.E
     return False
 
 
-def has_table_ancestor(element: TagsMixin) -> bool:
-    """Checks to see if an element has ancestors that are table elements. If so, we consider
-    it to be a table element rather than a section of narrative text."""
-    return any(ancestor in TABLE_TAGS for ancestor in element.ancestortags)
-
-
-def in_header_or_footer(element: TagsMixin) -> bool:
-    """Checks to see if an element is contained within a header or a footer tag."""
-    if any(ancestor in HEADER_OR_FOOTER_TAGS for ancestor in element.ancestortags):
-        return True
-    return False
-
-
-def _find_main(root: etree.Element) -> etree.Element:
-    """Finds the main tag of the HTML document if it exists. Otherwise, returns the
-    whole document."""
+def _find_main(root: etree._Element) -> etree._Element:
+    """The first <main> tag under `root` if it exists, othewise `root`."""
     main_tag_elem = root.find(".//main")
     return main_tag_elem if main_tag_elem is not None else root
 
 
-def _find_articles(root: etree.Element, assemble_articles: bool = True) -> List[etree.Element]:
-    """Tries to break the HTML document into distinct articles. If there are no article
-    tags, the entire document is returned as a single item list."""
+def _find_articles(root: etree._Element, assemble_articles: bool = True) -> List[etree._Element]:
+    """Parse articles from `root` of an HTML document.
+
+    Each `<article>` element in the HTML becomes its own "sub-document" (article). If no article
+    elements are present, the entire document (`root`) is returned as the single document article.
+    """
     if assemble_articles is False:
-        return root
+        return [root]
 
     articles = root.findall(".//article")
     if len(articles) == 0:
