@@ -1,22 +1,21 @@
 import os
+import typing as t
 import urllib.request
 from dataclasses import dataclass
 from ftplib import FTP, error_perm
 from pathlib import Path
-from typing import List, Optional
 
 import requests
 from bs4 import BeautifulSoup
 from requests.adapters import HTTPAdapter
-from urllib3.util import Retry
 
+from unstructured.ingest.error import SourceConnectionError, SourceConnectionNetworkError
 from unstructured.ingest.interfaces import (
-    BaseConnector,
     BaseConnectorConfig,
-    BaseIngestDoc,
-    ConnectorCleanupMixin,
+    BaseSingleIngestDoc,
+    BaseSourceConnector,
     IngestDocCleanupMixin,
-    StandardConnectorConfig,
+    SourceConnectorCleanupMixin,
 )
 from unstructured.ingest.logger import logger
 from unstructured.utils import (
@@ -41,14 +40,12 @@ class SimpleBiomedConfig(BaseConnectorConfig):
     """Connector config where path is the FTP directory path and
     id_, from_, until, format are API parameters."""
 
-    path: Optional[str]
+    path: t.Optional[str] = None
     # OA Web Service API Options
-    id_: Optional[str]
-    from_: Optional[str]
-    until: Optional[str]
-    max_retries: int = 5
+    id_: t.Optional[str] = None
+    from_: t.Optional[str] = None
+    until: t.Optional[str] = None
     request_timeout: int = 45
-    decay: float = 0.3
 
     def validate_api_inputs(self):
         valid = False
@@ -107,8 +104,8 @@ class SimpleBiomedConfig(BaseConnectorConfig):
 
 
 @dataclass
-class BiomedIngestDoc(IngestDocCleanupMixin, BaseIngestDoc):
-    config: SimpleBiomedConfig
+class BiomedIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
+    connector_config: SimpleBiomedConfig
     file_meta: BiomedFileMeta
     registry_name: str = "biomed"
 
@@ -122,14 +119,15 @@ class BiomedIngestDoc(IngestDocCleanupMixin, BaseIngestDoc):
 
     def cleanup_file(self):
         if (
-            not self.standard_config.preserve_downloads
+            not self.read_config.preserve_downloads
             and self.filename.is_file()
-            and not self.standard_config.download_only
+            and not self.read_config.download_only
         ):
             logger.debug(f"Cleaning up {self}")
             Path.unlink(self.filename)
 
-    @BaseIngestDoc.skip_if_file_exists
+    @SourceConnectionError.wrap
+    @BaseSingleIngestDoc.skip_if_file_exists
     def get_file(self):
         download_path = self.file_meta.download_filepath  # type: ignore
         dir_ = Path(os.path.dirname(download_path))  # type: ignore
@@ -138,26 +136,37 @@ class BiomedIngestDoc(IngestDocCleanupMixin, BaseIngestDoc):
 
             if dir_:
                 dir_.mkdir(parents=True, exist_ok=True)
+        self._retrieve()
+        logger.debug(f"File downloaded: {self.file_meta.download_filepath}")
+
+    @SourceConnectionNetworkError.wrap
+    def _retrieve(self):
         urllib.request.urlretrieve(
             self.file_meta.ftp_path,  # type: ignore
             self.file_meta.download_filepath,
         )
-        logger.debug(f"File downloaded: {self.file_meta.download_filepath}")
 
 
-class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
+class BiomedSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
     """Objects of this class support fetching documents from Biomedical literature FTP directory"""
 
-    config: SimpleBiomedConfig
+    connector_config: SimpleBiomedConfig
 
-    def __init__(
-        self,
-        standard_config: StandardConnectorConfig,
-        config: SimpleBiomedConfig,
-    ):
-        super().__init__(standard_config, config)
+    def get_base_endpoints_url(self) -> str:
+        endpoint_url = "https://www.ncbi.nlm.nih.gov/pmc/utils/oa/oa.fcgi?format=pdf"
 
-    def _list_objects_api(self):
+        if self.connector_config.id_:
+            endpoint_url += f"&id={self.connector_config.id_}"
+
+        if self.connector_config.from_:
+            endpoint_url += f"&from={self.connector_config.from_}"
+
+        if self.connector_config.until:
+            endpoint_url += f"&until={self.connector_config.until}"
+
+        return endpoint_url
+
+    def _list_objects_api(self) -> t.List[BiomedFileMeta]:
         def urls_to_metadata(urls):
             files = []
             for url in urls:
@@ -167,10 +176,10 @@ class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
                     files.append(
                         BiomedFileMeta(
                             ftp_path=url,
-                            download_filepath=(Path(self.standard_config.download_dir) / local_path)
+                            download_filepath=(Path(self.read_config.download_dir) / local_path)
                             .resolve()
                             .as_posix(),
-                            output_filepath=(Path(self.standard_config.output_dir) / local_path)
+                            output_filepath=(Path(self.processor_config.output_dir) / local_path)
                             .resolve()
                             .as_posix(),
                         ),
@@ -178,29 +187,16 @@ class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
 
             return files
 
-        files: List[BiomedFileMeta] = []
+        files: t.List[BiomedFileMeta] = []
 
-        endpoint_url = "https://www.ncbi.nlm.nih.gov/pmc/utils/oa/oa.fcgi?format=pdf"
-
-        if self.config.id_:
-            endpoint_url += f"&id={self.config.id_}"
-
-        if self.config.from_:
-            endpoint_url += f"&from={self.config.from_}"
-
-        if self.config.until:
-            endpoint_url += f"&until={self.config.until}"
+        endpoint_url = self.get_base_endpoints_url()
 
         while endpoint_url:
             session = requests.Session()
-            retries = Retry(
-                total=self.config.max_retries,
-                backoff_factor=self.config.decay,
-            )
-            adapter = HTTPAdapter(max_retries=retries)
+            adapter = HTTPAdapter()
             session.mount("http://", adapter)
             session.mount("https://", adapter)
-            response = session.get(endpoint_url, timeout=self.config.request_timeout)
+            response = self._get_request(session=session, endpoint_url=endpoint_url)
             soup = BeautifulSoup(response.content, features="lxml")
             urls = [link["href"] for link in soup.find_all("link")]
 
@@ -215,12 +211,16 @@ class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
 
         return files
 
-    def _list_objects(self):
+    @SourceConnectionNetworkError.wrap
+    def _get_request(self, session: requests.Session, endpoint_url: str) -> requests.Response:
+        return session.get(endpoint_url, timeout=self.connector_config.request_timeout)
+
+    def _list_objects(self) -> t.List[BiomedFileMeta]:
         files = []
 
         # Conform to mypy, null check performed elsewhere.
         # Wouldn't be in this method unless self.config.path exists
-        path: str = self.config.path if self.config.path else ""
+        path: str = self.connector_config.path if self.connector_config.path else ""
 
         def traverse(path, download_dir, output_dir):
             full_path = Path(PMC_DIR) / path
@@ -248,12 +248,12 @@ class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
                         files.append(
                             BiomedFileMeta(
                                 ftp_path=ftp_path,
-                                download_filepath=(
-                                    Path(self.standard_config.download_dir) / local_path
-                                )
+                                download_filepath=(Path(self.read_config.download_dir) / local_path)
                                 .resolve()
                                 .as_posix(),
-                                output_filepath=(Path(self.standard_config.output_dir) / local_path)
+                                output_filepath=(
+                                    Path(self.processor_config.output_dir) / local_path
+                                )
                                 .resolve()
                                 .as_posix(),
                             ),
@@ -266,16 +266,16 @@ class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
             else:
                 raise ValueError(f"{full_path} is not a valid directory.")
 
-        ftp_path = f"{FTP_DOMAIN}/{PMC_DIR}/{self.config.path}"
-        if self.config.is_file:
+        ftp_path = f"{FTP_DOMAIN}/{PMC_DIR}/{self.connector_config.path}"
+        if self.connector_config.is_file:
             local_path = "/".join(path.split("/")[1:])
             return [
                 BiomedFileMeta(
                     ftp_path=ftp_path,
-                    download_filepath=(Path(self.standard_config.download_dir) / local_path)
+                    download_filepath=(Path(self.read_config.download_dir) / local_path)
                     .resolve()
                     .as_posix(),
-                    output_filepath=(Path(self.standard_config.output_dir) / local_path)
+                    output_filepath=(Path(self.processor_config.output_dir) / local_path)
                     .resolve()
                     .as_posix(),
                 ),
@@ -283,8 +283,8 @@ class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
         else:
             traverse(
                 Path(path),
-                Path(self.standard_config.download_dir),
-                Path(self.standard_config.output_dir),
+                Path(self.read_config.download_dir),
+                Path(self.processor_config.output_dir),
             )
 
         return files
@@ -292,6 +292,21 @@ class BiomedConnector(ConnectorCleanupMixin, BaseConnector):
     def initialize(self):
         pass
 
+    def check_connection(self):
+        resp = requests.head(self.get_base_endpoints_url())
+        try:
+            resp.raise_for_status()
+        except requests.HTTPError as http_error:
+            raise SourceConnectionError(f"failed to validate connection: {http_error}")
+
     def get_ingest_docs(self):
-        files = self._list_objects_api() if self.config.is_api else self._list_objects()
-        return [BiomedIngestDoc(self.standard_config, self.config, file) for file in files]
+        files = self._list_objects_api() if self.connector_config.is_api else self._list_objects()
+        return [
+            BiomedIngestDoc(
+                processor_config=self.processor_config,
+                connector_config=self.connector_config,
+                read_config=self.read_config,
+                file_meta=file,
+            )
+            for file in files
+        ]
