@@ -2,11 +2,12 @@ from dataclasses import dataclass
 from typing import Type
 
 from unstructured.ingest.connector.fsspec import (
-    FsspecConnector,
+    FsspecDestinationConnector,
     FsspecIngestDoc,
+    FsspecSourceConnector,
     SimpleFsspecConfig,
 )
-from unstructured.ingest.interfaces import StandardConnectorConfig
+from unstructured.ingest.error import SourceConnectionError
 from unstructured.utils import requires_dependencies
 
 
@@ -17,21 +18,24 @@ class SimpleGcsConfig(SimpleFsspecConfig):
 
 @dataclass
 class GcsIngestDoc(FsspecIngestDoc):
-    config: SimpleGcsConfig
+    connector_config: SimpleGcsConfig
     registry_name: str = "gcs"
 
+    @SourceConnectionError.wrap
     @requires_dependencies(["gcsfs", "fsspec"], extras="gcs")
     def get_file(self):
         super().get_file()
 
 
-@requires_dependencies(["gcsfs", "fsspec"], extras="gcs")
-class GcsConnector(FsspecConnector):
-    ingest_doc_cls: Type[GcsIngestDoc] = GcsIngestDoc
+@dataclass
+class GcsSourceConnector(FsspecSourceConnector):
+    connector_config: SimpleGcsConfig
 
-    def __init__(
-        self,
-        config: SimpleGcsConfig,
-        standard_config: StandardConnectorConfig,
-    ) -> None:
-        super().__init__(standard_config, config)
+    def __post_init__(self):
+        self.ingest_doc_cls: Type[GcsIngestDoc] = GcsIngestDoc
+
+
+@requires_dependencies(["gcsfs", "fsspec"], extras="gcs")
+@dataclass
+class GcsDestinationConnector(FsspecDestinationConnector):
+    connector_config: SimpleGcsConfig
