@@ -1,29 +1,32 @@
 import io
 import json
 import os
-from dataclasses import dataclass
+import typing as t
+from dataclasses import dataclass, field
+from datetime import datetime
 from mimetypes import guess_extension
 from pathlib import Path
-from typing import TYPE_CHECKING, Dict, Optional
 
 from unstructured.file_utils.filetype import EXT_TO_FILETYPE
 from unstructured.file_utils.google_filetype import GOOGLE_DRIVE_EXPORT_TYPES
+from unstructured.ingest.error import SourceConnectionError, SourceConnectionNetworkError
 from unstructured.ingest.interfaces import (
-    BaseConnector,
     BaseConnectorConfig,
-    BaseIngestDoc,
     BaseSessionHandle,
+    BaseSingleIngestDoc,
+    BaseSourceConnector,
     ConfigSessionHandleMixin,
-    ConnectorCleanupMixin,
     IngestDocCleanupMixin,
     IngestDocSessionHandleMixin,
-    StandardConnectorConfig,
+    SourceConnectorCleanupMixin,
+    SourceMetadata,
 )
 from unstructured.ingest.logger import logger
 from unstructured.utils import requires_dependencies
 
-if TYPE_CHECKING:
+if t.TYPE_CHECKING:
     from googleapiclient.discovery import Resource as GoogleAPIResource
+    from googleapiclient.http import MediaIoBaseDownload
 
 FILE_FORMAT = "{id}-{name}{ext}"
 DIRECTORY_FORMAT = "{id}-{name}"
@@ -35,7 +38,7 @@ class GoogleDriveSessionHandle(BaseSessionHandle):
 
 
 @requires_dependencies(["googleapiclient"], extras="google-drive")
-def create_service_account_object(key_path, id=None):
+def create_service_account_object(key_path: t.Union[str, dict], id=None):
     """
     Creates a service object for interacting with Google Drive.
 
@@ -50,12 +53,21 @@ def create_service_account_object(key_path, id=None):
         Service account object
     """
     from google.auth import default, exceptions
+    from google.oauth2 import service_account
     from googleapiclient.discovery import build
     from googleapiclient.errors import HttpError
 
     try:
-        os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = key_path
-        creds, _ = default()
+        if isinstance(key_path, dict):
+            creds = service_account.Credentials.from_service_account_info(key_path)
+        elif isinstance(key_path, str):
+            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = key_path
+            creds, _ = default()
+        else:
+            raise ValueError(
+                f"key path not recognized as a dictionary or a file path: "
+                f"[{type(key_path)}] {key_path}",
+            )
         service = build("drive", "v3", credentials=creds)
 
         if id:
@@ -82,8 +94,8 @@ class SimpleGoogleDriveConfig(ConfigSessionHandleMixin, BaseConnectorConfig):
 
     # Google Drive Specific Options
     drive_id: str
-    service_account_key: str
-    extension: Optional[str]
+    service_account_key: t.Union[str, dict]
+    extension: t.Optional[str] = None
     recursive: bool = False
 
     def __post_init__(self):
@@ -101,58 +113,112 @@ class SimpleGoogleDriveConfig(ConfigSessionHandleMixin, BaseConnectorConfig):
 
 
 @dataclass
-class GoogleDriveIngestDoc(IngestDocSessionHandleMixin, IngestDocCleanupMixin, BaseIngestDoc):
-    config: SimpleGoogleDriveConfig
-    file_meta: Dict[str, str]
+class GoogleDriveIngestDoc(IngestDocSessionHandleMixin, IngestDocCleanupMixin, BaseSingleIngestDoc):
+    connector_config: SimpleGoogleDriveConfig
+    meta: t.Dict[str, str] = field(default_factory=dict)
     registry_name: str = "google_drive"
 
     @property
     def filename(self):
-        return Path(self.file_meta.get("download_filepath")).resolve()  # type: ignore
+        return Path(self.meta.get("download_filepath")).resolve()  # type: ignore
 
     @property
     def _output_filename(self):
-        return Path(f"{self.file_meta.get('output_filepath')}.json").resolve()
+        return Path(f"{self.meta.get('output_filepath')}.json").resolve()
+
+    @property
+    def record_locator(self) -> t.Optional[t.Dict[str, t.Any]]:
+        return {
+            "drive_id": self.connector_config.drive_id,
+            "file_id": self.meta["id"],
+        }
 
-    @BaseIngestDoc.skip_if_file_exists
     @requires_dependencies(["googleapiclient"], extras="google-drive")
-    def get_file(self):
+    def update_source_metadata(self):
         from googleapiclient.errors import HttpError
+
+        try:
+            file_obj = (
+                self.session_handle.service.files()
+                .get(
+                    fileId=self.meta["id"],
+                    fields="id, createdTime, modifiedTime, version, webContentLink",
+                )
+                .execute()
+            )
+        except HttpError as e:
+            if e.status_code == 404:
+                logger.error(f"File {self.meta['name']} not found")
+                self.source_metadata = SourceMetadata(
+                    exists=True,
+                )
+                return
+            raise
+
+        date_created = None
+        if dc := file_obj.get("createdTime", ""):
+            date_created = datetime.strptime(
+                dc,
+                "%Y-%m-%dT%H:%M:%S.%fZ",
+            ).isoformat()
+
+        date_modified = None
+        if dm := file_obj.get("modifiedTime", ""):
+            date_modified = datetime.strptime(
+                dm,
+                "%Y-%m-%dT%H:%M:%S.%fZ",
+            ).isoformat()
+
+        self.source_metadata = SourceMetadata(
+            date_created=date_created,
+            date_modified=date_modified,
+            version=file_obj.get("version", ""),
+            source_url=file_obj.get("webContentLink", ""),
+            exists=True,
+        )
+
+    @SourceConnectionNetworkError.wrap
+    def _run_downloader(self, downloader: "MediaIoBaseDownload") -> bool:
+        downloaded = False
+        while downloaded is False:
+            _, downloaded = downloader.next_chunk()
+        return downloaded
+
+    @requires_dependencies(["googleapiclient"], extras="google-drive")
+    @SourceConnectionError.wrap
+    @BaseSingleIngestDoc.skip_if_file_exists
+    def get_file(self):
         from googleapiclient.http import MediaIoBaseDownload
 
-        if self.file_meta.get("mimeType", "").startswith("application/vnd.google-apps"):
+        if self.meta.get("mimeType", "").startswith("application/vnd.google-apps"):
             export_mime = GOOGLE_DRIVE_EXPORT_TYPES.get(
-                self.file_meta.get("mimeType"),  # type: ignore
+                self.meta.get("mimeType"),  # type: ignore
             )
             if not export_mime:
                 logger.info(
-                    f"File not supported. Name: {self.file_meta.get('name')} "
-                    f"ID: {self.file_meta.get('id')} "
-                    f"MimeType: {self.file_meta.get('mimeType')}",
+                    f"File not supported. Name: {self.meta.get('name')} "
+                    f"ID: {self.meta.get('id')} "
+                    f"MimeType: {self.meta.get('mimeType')}",
                 )
                 return
 
             request = self.session_handle.service.files().export_media(
-                fileId=self.file_meta.get("id"),
+                fileId=self.meta.get("id"),
                 mimeType=export_mime,
             )
         else:
-            request = self.session_handle.service.files().get_media(fileId=self.file_meta.get("id"))
+            request = self.session_handle.service.files().get_media(fileId=self.meta.get("id"))
         file = io.BytesIO()
         downloader = MediaIoBaseDownload(file, request)
-        downloaded = False
-        try:
-            while downloaded is False:
-                status, downloaded = downloader.next_chunk()
-        except HttpError:
-            pass
+        self.update_source_metadata()
+        downloaded = self._run_downloader(downloader=downloader)
 
         saved = False
         if downloaded and file:
-            dir_ = Path(self.file_meta["download_dir"])
+            dir_ = Path(self.meta["download_dir"])
             if dir_:
                 if not dir_.is_dir():
-                    logger.debug(f"Creating directory: {self.file_meta.get('download_dir')}")
+                    logger.debug(f"Creating directory: {self.meta.get('download_dir')}")
 
                     if dir_:
                         dir_.mkdir(parents=True, exist_ok=True)
@@ -161,13 +227,12 @@ class GoogleDriveIngestDoc(IngestDocSessionHandleMixin, IngestDocCleanupMixin, B
                     handler.write(file.getbuffer())
                     saved = True
                     logger.debug(f"File downloaded: {self.filename}.")
-
         if not saved:
             logger.error(f"Error while downloading and saving file: {self.filename}.")
 
     def write_result(self):
         """Write the structured json result for this doc. result must be json serializable."""
-        if self.standard_config.download_only:
+        if self.read_config.download_only:
             return
         self._output_filename.parent.mkdir(parents=True, exist_ok=True)
         with open(self._output_filename, "w") as output_f:
@@ -175,17 +240,15 @@ class GoogleDriveIngestDoc(IngestDocSessionHandleMixin, IngestDocCleanupMixin, B
         logger.info(f"Wrote {self._output_filename}")
 
 
-class GoogleDriveConnector(ConnectorCleanupMixin, BaseConnector):
+@dataclass
+class GoogleDriveSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
     """Objects of this class support fetching documents from Google Drive"""
 
-    config: SimpleGoogleDriveConfig
-
-    def __init__(self, standard_config: StandardConnectorConfig, config: SimpleGoogleDriveConfig):
-        super().__init__(standard_config, config)
+    connector_config: SimpleGoogleDriveConfig
 
     def _list_objects(self, drive_id, recursive=False):
         files = []
-        service = self.config.create_session_handle().service
+        service = self.connector_config.create_session_handle().service
 
         def traverse(drive_id, download_dir, output_dir, recursive=False):
             page_token = None
@@ -229,10 +292,13 @@ class GoogleDriveConnector(ConnectorCleanupMixin, BaseConnector):
                                 ext = guess if guess else ext
 
                         # TODO (Habeeb): Consider filtering at the query level.
-                        if self.config.extension and self.config.extension != ext:  # noqa: SIM102
+                        if (
+                            self.connector_config.extension
+                            and self.connector_config.extension != ext
+                        ):  # noqa: SIM102
                             logger.debug(
                                 f"File {meta.get('name')} does not match "
-                                f"the file type {self.config.extension}",
+                                f"the file type {self.connector_config.extension}",
                             )
                             continue
 
@@ -249,8 +315,8 @@ class GoogleDriveConnector(ConnectorCleanupMixin, BaseConnector):
 
         traverse(
             drive_id,
-            Path(self.standard_config.download_dir),
-            Path(self.standard_config.output_dir),
+            Path(self.read_config.download_dir),
+            Path(self.processor_config.output_dir),
             recursive,
         )
         return files
@@ -258,6 +324,21 @@ class GoogleDriveConnector(ConnectorCleanupMixin, BaseConnector):
     def initialize(self):
         pass
 
+    def check_connection(self):
+        try:
+            self.connector_config.create_session_handle().service
+        except Exception as e:
+            logger.error(f"failed to validate connection: {e}", exc_info=True)
+            raise SourceConnectionError(f"failed to validate connection: {e}")
+
     def get_ingest_docs(self):
-        files = self._list_objects(self.config.drive_id, self.config.recursive)
-        return [GoogleDriveIngestDoc(self.standard_config, self.config, file) for file in files]
+        files = self._list_objects(self.connector_config.drive_id, self.connector_config.recursive)
+        return [
+            GoogleDriveIngestDoc(
+                connector_config=self.connector_config,
+                processor_config=self.processor_config,
+                read_config=self.read_config,
+                meta=file,
+            )
+            for file in files
+        ]
