@@ -2,7 +2,7 @@ from abc import ABC
 from functools import wraps
 
 
-class CustomError(BaseException, ABC):
+class CustomError(Exception, ABC):
     error_string: str
 
     @classmethod
@@ -18,14 +18,18 @@ class CustomError(BaseException, ABC):
             try:
                 return f(*args, **kwargs)
             except BaseException as error:
-                if not isinstance(error, cls):
-                    raise cls(cls.error_string.format(str(error)))
-                raise error
+                if not isinstance(error, cls) and not issubclass(type(error), cls):
+                    raise cls(cls.error_string.format(str(error))) from error
+                raise
 
         return wrapper
 
 
 class SourceConnectionError(CustomError):
+    error_string = "Error in getting data from upstream data source: {}"
+
+
+class SourceConnectionNetworkError(SourceConnectionError):
     error_string = "Error in connecting to upstream data source: {}"
 
 
@@ -33,5 +37,13 @@ class DestinationConnectionError(CustomError):
     error_string = "Error in connecting to downstream data source: {}"
 
 
+class EmbeddingEncoderConnectionError(CustomError):
+    error_string = "Error in connecting to the embedding model provider: {}"
+
+
+class WriteError(CustomError):
+    error_string = "Error in writing to downstream data source: {}"
+
+
 class PartitionError(CustomError):
     error_string = "Error in partitioning content: {}"
