@@ -16,7 +16,11 @@ from unstructured.partition.common import exactly_one
 from unstructured.partition.email import partition_email
 from unstructured.partition.html import partition_html
 from unstructured.partition.json import partition_json
+from unstructured.partition.lang import (
+    convert_old_ocr_languages_to_languages,
+)
 from unstructured.partition.text import partition_text
+from unstructured.partition.utils.constants import PartitionStrategy
 from unstructured.partition.xml import partition_xml
 from unstructured.utils import dependency_exists
 
@@ -44,7 +48,7 @@ if dependency_exists("docx") and dependency_exists("pypandoc"):
     PARTITION_WITH_EXTRAS_MAP["odt"] = partition_odt
 
 
-if dependency_exists("ebooklib"):
+if dependency_exists("pypandoc"):
     from unstructured.partition.epub import partition_epub
 
     PARTITION_WITH_EXTRAS_MAP["epub"] = partition_epub
@@ -122,17 +126,22 @@ def partition(
     file_filename: Optional[str] = None,
     url: Optional[str] = None,
     include_page_breaks: bool = False,
-    strategy: str = "auto",
+    strategy: str = PartitionStrategy.AUTO,
     encoding: Optional[str] = None,
     paragraph_grouper: Optional[Callable[[str], str]] = None,
     headers: Dict[str, str] = {},
-    skip_infer_table_types: List[str] = ["pdf", "jpg", "png"],
+    skip_infer_table_types: List[str] = ["pdf", "jpg", "png", "xls", "xlsx"],
     ssl_verify: bool = True,
-    ocr_languages: str = "eng",
+    ocr_languages: Optional[str] = None,  # changing to optional for deprecation
+    languages: Optional[List[str]] = None,
+    detect_language_per_element: bool = False,
     pdf_infer_table_structure: bool = False,
+    pdf_extract_images: bool = False,
+    pdf_image_output_dir_path: Optional[str] = None,
     xml_keep_tags: bool = False,
     data_source_metadata: Optional[DataSourceMetadata] = None,
     metadata_filename: Optional[str] = None,
+    request_timeout: Optional[int] = None,
     **kwargs,
 ):
     """Partitions a document into its constituent elements. Will use libmagic to determine
@@ -168,17 +177,31 @@ def partition(
     ssl_verify
         If the URL parameter is set, determines whether or not partition uses SSL verification
         in the HTTP request.
-    ocr_languages
-        The languages to use for the Tesseract agent. To use a language, you'll first need
-        to isntall the appropriate Tesseract language pack.
+    languages
+        The languages present in the document, for use in partitioning and/or OCR. For partitioning
+        image or pdf documents with Tesseract, you'll first need to install the appropriate
+        Tesseract language pack. For other partitions, language is detected using naive Bayesian
+        filter via `langdetect`. Multiple languages indicates text could be in either language.
+        Additional Parameters:
+            detect_language_per_element
+                Detect language per element instead of at the document level.
     pdf_infer_table_structure
         If True and strategy=hi_res, any Table Elements extracted from a PDF will include an
         additional metadata field, "text_as_html," where the value (string) is a just a
         transformation of the data into an HTML <table>.
         The "text" field for a partitioned Table Element is always present, whether True or False.
+    pdf_extract_images
+        If True and strategy=hi_res, any detected images will be saved in the path specified by
+        pdf_image_output_dir_path.
+    pdf_image_output_dir_path
+        If pdf_extract_images=True and strategy=hi_res, any detected images will be saved in the
+        given path
     xml_keep_tags
         If True, will retain the XML tags in the output. Otherwise it will simply extract
         the text from within the tags. Only applies to partition_xml.
+    request_timeout
+        The timeout for the HTTP request if URL is set. Defaults to None meaning no timeout and
+        requests will block indefinitely.
     """
     exactly_one(file=file, filename=filename, url=url)
 
@@ -196,12 +219,31 @@ def partition(
         )
     kwargs.setdefault("metadata_filename", metadata_filename)
 
+    if ocr_languages == "":
+        ocr_languages = None
+
+    if ocr_languages is not None:
+        # check if languages was set to anything not the default value
+        # languages and ocr_languages were therefore both provided - raise error
+        if languages is not None:
+            raise ValueError(
+                "Only one of languages and ocr_languages should be specified. "
+                "languages is preferred. ocr_languages is marked for deprecation.",
+            )
+        else:
+            languages = convert_old_ocr_languages_to_languages(ocr_languages)
+            logger.warning(
+                "The ocr_languages kwarg will be deprecated in a future version of unstructured. "
+                "Please use languages instead.",
+            )
+
     if url is not None:
         file, filetype = file_and_type_from_url(
             url=url,
             content_type=content_type,
             headers=headers,
             ssl_verify=ssl_verify,
+            request_timeout=request_timeout,
         )
     else:
         if headers != {}:
@@ -228,24 +270,60 @@ def partition(
 
     if filetype == FileType.DOC:
         _partition_doc = _get_partition_with_extras("doc")
-        elements = _partition_doc(filename=filename, file=file, **kwargs)
+        elements = _partition_doc(
+            filename=filename,
+            file=file,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.DOCX:
         _partition_docx = _get_partition_with_extras("docx")
-        elements = _partition_docx(filename=filename, file=file, **kwargs)
+        elements = _partition_docx(
+            filename=filename,
+            file=file,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.ODT:
         _partition_odt = _get_partition_with_extras("odt")
-        elements = _partition_odt(filename=filename, file=file, **kwargs)
+        elements = _partition_odt(
+            filename=filename,
+            file=file,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.EML:
-        elements = partition_email(filename=filename, file=file, encoding=encoding, **kwargs)
+        elements = partition_email(
+            filename=filename,
+            file=file,
+            encoding=encoding,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.MSG:
         _partition_msg = _get_partition_with_extras("msg")
-        elements = _partition_msg(filename=filename, file=file, **kwargs)
+        elements = _partition_msg(
+            filename=filename,
+            file=file,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.HTML:
         elements = partition_html(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
             encoding=encoding,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.XML:
@@ -254,6 +332,8 @@ def partition(
             file=file,
             encoding=encoding,
             xml_keep_tags=xml_keep_tags,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.EPUB:
@@ -262,6 +342,9 @@ def partition(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.ORG:
@@ -270,6 +353,8 @@ def partition(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.RST:
@@ -278,6 +363,9 @@ def partition(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.MD:
@@ -286,6 +374,9 @@ def partition(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.PDF:
@@ -297,7 +388,9 @@ def partition(
             include_page_breaks=include_page_breaks,
             infer_table_structure=infer_table_structure,
             strategy=strategy,
-            ocr_languages=ocr_languages,
+            languages=languages,
+            extract_images_in_pdf=pdf_extract_images,
+            image_output_dir_path=pdf_image_output_dir_path,
             **kwargs,
         )
     elif (filetype == FileType.PNG) or (filetype == FileType.JPG) or (filetype == FileType.TIFF):
@@ -308,7 +401,7 @@ def partition(
             include_page_breaks=include_page_breaks,
             infer_table_structure=infer_table_structure,
             strategy=strategy,
-            ocr_languages=ocr_languages,
+            languages=languages,
             **kwargs,
         )
     elif filetype == FileType.TXT:
@@ -317,6 +410,8 @@ def partition(
             file=file,
             encoding=encoding,
             paragraph_grouper=paragraph_grouper,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.RTF:
@@ -325,6 +420,9 @@ def partition(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.PPT:
@@ -333,6 +431,9 @@ def partition(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.PPTX:
@@ -341,6 +442,9 @@ def partition(
             filename=filename,
             file=file,
             include_page_breaks=include_page_breaks,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
             **kwargs,
         )
     elif filetype == FileType.JSON:
@@ -352,13 +456,33 @@ def partition(
         elements = partition_json(filename=filename, file=file, **kwargs)
     elif (filetype == FileType.XLSX) or (filetype == FileType.XLS):
         _partition_xlsx = _get_partition_with_extras("xlsx")
-        elements = _partition_xlsx(filename=filename, file=file, **kwargs)
+        elements = _partition_xlsx(
+            filename=filename,
+            file=file,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.CSV:
         _partition_csv = _get_partition_with_extras("csv")
-        elements = _partition_csv(filename=filename, file=file, **kwargs)
+        elements = _partition_csv(
+            filename=filename,
+            file=file,
+            infer_table_structure=infer_table_structure,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.TSV:
         _partition_tsv = _get_partition_with_extras("tsv")
-        elements = _partition_tsv(filename=filename, file=file, **kwargs)
+        elements = _partition_tsv(
+            filename=filename,
+            file=file,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+            **kwargs,
+        )
     elif filetype == FileType.EMPTY:
         elements = []
     else:
@@ -384,8 +508,9 @@ def file_and_type_from_url(
     content_type: Optional[str] = None,
     headers: Dict[str, str] = {},
     ssl_verify: bool = True,
+    request_timeout: Optional[int] = None,
 ) -> Tuple[io.BytesIO, Optional[FileType]]:
-    response = requests.get(url, headers=headers, verify=ssl_verify)
+    response = requests.get(url, headers=headers, verify=ssl_verify, timeout=request_timeout)
     file = io.BytesIO(response.content)
 
     content_type = content_type or response.headers.get("Content-Type")
@@ -409,6 +534,6 @@ def decide_table_extraction(
                 f"and pdf_infer_table_structure: {pdf_infer_table_structure}, "
                 "please reset skip_infer_table_types to turn on table extraction for PDFs.",
             )
-        return not (doc_type in skip_infer_table_types) or pdf_infer_table_structure
+        return doc_type not in skip_infer_table_types or pdf_infer_table_structure
 
-    return not (doc_type in skip_infer_table_types)
+    return doc_type not in skip_infer_table_types
