@@ -1,11 +1,22 @@
 from __future__ import annotations
 
+import numbers
 import os
 import subprocess
 from datetime import datetime
 from io import BufferedReader, BytesIO, TextIOWrapper
 from tempfile import SpooledTemporaryFile
-from typing import IO, TYPE_CHECKING, Any, BinaryIO, Dict, List, Optional, Tuple, Union
+from typing import (
+    IO,
+    TYPE_CHECKING,
+    Any,
+    BinaryIO,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Union,
+)
 
 import emoji
 from tabulate import tabulate
@@ -17,27 +28,56 @@ from unstructured.documents.elements import (
     CoordinatesMetadata,
     Element,
     ElementMetadata,
+    ElementType,
     ListItem,
     PageBreak,
     Text,
+    Title,
 )
 from unstructured.logger import logger
 from unstructured.nlp.patterns import ENUMERATED_BULLETS_RE, UNICODE_BULLETS_RE
-from unstructured.partition.utils.constants import SORT_MODE_XY_CUT
-from unstructured.utils import dependency_exists
+from unstructured.partition.utils.constants import SORT_MODE_DONT, SORT_MODE_XY_CUT
+from unstructured.utils import dependency_exists, first
 
 if dependency_exists("docx") and dependency_exists("docx.table"):
     from docx.table import Table as docxtable
 
+if dependency_exists("pptx") and dependency_exists("pptx.table"):
+    from pptx.table import Table as pptxtable
+
 if dependency_exists("numpy") and dependency_exists("cv2"):
     from unstructured.partition.utils.sorting import sort_page_elements
 
 if TYPE_CHECKING:
     from unstructured_inference.inference.layout import DocumentLayout, PageLayout
-    from unstructured_inference.inference.layoutelement import (
-        LayoutElement,
-        LocationlessLayoutElement,
-    )
+    from unstructured_inference.inference.layoutelement import LayoutElement
+
+
+HIERARCHY_RULE_SET = {
+    "Title": [
+        "Text",
+        "UncategorizedText",
+        "NarrativeText",
+        "ListItem",
+        "BulletedText",
+        "Table",
+        "FigureCaption",
+        "CheckBox",
+        "Table",
+    ],
+    "Header": [
+        "Title",
+        "Text",
+        "UncategorizedText",
+        "NarrativeText",
+        "ListItem",
+        "BulletedText",
+        "Table",
+        "FigureCaption",
+        "CheckBox",
+        "Table",
+    ],
+}
 
 
 def get_last_modified_date(filename: str) -> Union[str, None]:
@@ -46,7 +86,7 @@ def get_last_modified_date(filename: str) -> Union[str, None]:
 
 
 def get_last_modified_date_from_file(
-    file: Union[IO[bytes], SpooledTemporaryFile, BinaryIO, bytes],
+    file: Union[IO[bytes], SpooledTemporaryFile[bytes], BinaryIO, bytes],
 ) -> Union[str, None]:
     filename = None
     if hasattr(file, "name"):
@@ -62,16 +102,16 @@ def get_last_modified_date_from_file(
 def normalize_layout_element(
     layout_element: Union[
         "LayoutElement",
-        "LocationlessLayoutElement",
         Element,
         Dict[str, Any],
     ],
     coordinate_system: Optional[CoordinateSystem] = None,
     infer_list_items: bool = True,
+    source_format: Optional[str] = "html",
 ) -> Union[Element, List[Element]]:
     """Converts an unstructured_inference LayoutElement object to an unstructured Element."""
 
-    if isinstance(layout_element, Element):
+    if isinstance(layout_element, Element) and source_format == "html":
         return layout_element
 
     # NOTE(alan): Won't the lines above ensure this never runs (PageBreak is a subclass of Element)?
@@ -88,74 +128,152 @@ def normalize_layout_element(
     # in order to add coordinates metadata to the element.
     coordinates = layout_dict.get("coordinates")
     element_type = layout_dict.get("type")
-    if element_type == "List":
+    prob = layout_dict.get("prob")
+    aux_origin = layout_dict["source"] if "source" in layout_dict else None
+    origin = None
+    if aux_origin:
+        origin = aux_origin.value
+    if prob and isinstance(prob, (int, str, float, numbers.Number)):
+        class_prob_metadata = ElementMetadata(detection_class_prob=float(prob))  # type: ignore
+    else:
+        class_prob_metadata = ElementMetadata()
+    if element_type == ElementType.LIST:
         if infer_list_items:
             return layout_list_to_list_items(
                 text,
                 coordinates=coordinates,
                 coordinate_system=coordinate_system,
+                metadata=class_prob_metadata,
+                detection_origin=origin,
             )
         else:
             return ListItem(
-                text=text,
+                text=text if text else "",
                 coordinates=coordinates,
                 coordinate_system=coordinate_system,
+                metadata=class_prob_metadata,
+                detection_origin=origin,
             )
 
     elif element_type in TYPE_TO_TEXT_ELEMENT_MAP:
         _element_class = TYPE_TO_TEXT_ELEMENT_MAP[element_type]
-        return _element_class(
+        _element_class = _element_class(
             text=text,
             coordinates=coordinates,
             coordinate_system=coordinate_system,
+            metadata=class_prob_metadata,
+            detection_origin=origin,
         )
-    elif element_type == "Checked":
+        if element_type == ElementType.HEADLINE:
+            _element_class.metadata.category_depth = 1
+        elif element_type == ElementType.SUB_HEADLINE:
+            _element_class.metadata.category_depth = 2
+        return _element_class
+    elif element_type == ElementType.CHECKED:
         return CheckBox(
             checked=True,
             coordinates=coordinates,
             coordinate_system=coordinate_system,
+            metadata=class_prob_metadata,
+            detection_origin=origin,
         )
-    elif element_type == "Unchecked":
+    elif element_type == ElementType.UNCHECKED:
         return CheckBox(
             checked=False,
             coordinates=coordinates,
             coordinate_system=coordinate_system,
+            metadata=class_prob_metadata,
+            detection_origin=origin,
         )
     else:
         return Text(
-            text=text,
+            text=text if text else "",
             coordinates=coordinates,
             coordinate_system=coordinate_system,
+            metadata=class_prob_metadata,
+            detection_origin=origin,
         )
 
 
 def layout_list_to_list_items(
-    text: str,
-    coordinates: Tuple[Tuple[float, float], ...],
+    text: Optional[str],
+    coordinates: Optional[Tuple[Tuple[float, float], ...]],
     coordinate_system: Optional[CoordinateSystem],
+    metadata=Optional[ElementMetadata],
+    detection_origin=Optional[str],
 ) -> List[Element]:
     """Converts a list LayoutElement to a list of ListItem elements."""
-    split_items = ENUMERATED_BULLETS_RE.split(text)
+    split_items = ENUMERATED_BULLETS_RE.split(text) if text else []
     # NOTE(robinson) - this means there wasn't a match for the enumerated bullets
     if len(split_items) == 1:
-        split_items = UNICODE_BULLETS_RE.split(text)
+        split_items = UNICODE_BULLETS_RE.split(text) if text else []
 
     list_items: List[Element] = []
     for text_segment in split_items:
         if len(text_segment.strip()) > 0:
             # Both `coordinates` and `coordinate_system` must be present
             # in order to add coordinates metadata to the element.
-            list_items.append(
-                ListItem(
-                    text=text_segment.strip(),
-                    coordinates=coordinates,
-                    coordinate_system=coordinate_system,
-                ),
+            item = ListItem(
+                text=text_segment.strip(),
+                coordinates=coordinates,
+                coordinate_system=coordinate_system,
+                metadata=metadata,
+                detection_origin=detection_origin,
             )
+            list_items.append(item)
 
     return list_items
 
 
+def set_element_hierarchy(
+    elements: List[Element],
+    ruleset: Dict[str, List[str]] = HIERARCHY_RULE_SET,
+) -> List[Element]:
+    """Sets the parent_id for each element in the list of elements
+    based on the element's category, depth and a ruleset
+
+    """
+    stack: List[Element] = []
+    for element in elements:
+        if element.metadata.parent_id is not None:
+            continue
+        parent_id = None
+        element_category = getattr(element, "category", None)
+        element_category_depth = getattr(element.metadata, "category_depth", 0) or 0
+
+        if not element_category:
+            continue
+
+        while stack:
+            top_element: Element = stack[-1]
+            top_element_category = getattr(top_element, "category")
+            top_element_category_depth = (
+                getattr(
+                    top_element.metadata,
+                    "category_depth",
+                    0,
+                )
+                or 0
+            )
+
+            if (
+                top_element_category == element_category
+                and top_element_category_depth < element_category_depth
+            ) or (
+                top_element_category != element_category
+                and element_category in ruleset.get(top_element_category, [])
+            ):
+                parent_id = top_element.id
+                break
+
+            stack.pop()
+
+        element.metadata.parent_id = parent_id
+        stack.append(element)
+
+    return elements
+
+
 def _add_element_metadata(
     element: Element,
     filename: Optional[str] = None,
@@ -166,10 +284,14 @@ def _add_element_metadata(
     coordinates: Optional[Tuple[Tuple[float, float], ...]] = None,
     coordinate_system: Optional[CoordinateSystem] = None,
     section: Optional[str] = None,
+    image_path: Optional[str] = None,
+    detection_origin: Optional[str] = None,
+    languages: Optional[List[str]] = None,
     **kwargs,
 ) -> Element:
     """Adds document metadata to the document element. Document metadata includes information
     like the filename, source url, and page number."""
+
     coordinates_metadata = (
         CoordinatesMetadata(
             points=coordinates,
@@ -196,6 +318,8 @@ def _add_element_metadata(
         if emphasized_texts
         else None
     )
+    depth = element.metadata.category_depth if element.metadata.category_depth else None
+
     metadata = ElementMetadata(
         coordinates=coordinates_metadata,
         filename=filename,
@@ -208,8 +332,13 @@ def _add_element_metadata(
         emphasized_text_contents=emphasized_text_contents,
         emphasized_text_tags=emphasized_text_tags,
         section=section,
+        category_depth=depth,
+        image_path=image_path,
+        languages=languages,
     )
-    element.metadata = metadata.merge(element.metadata)
+    element.metadata.update(metadata)
+    if detection_origin is not None:
+        element.metadata.detection_origin = detection_origin
     return element
 
 
@@ -296,7 +425,7 @@ on your system and try again.
         logger.error(error.decode().strip())
 
 
-def exactly_one(**kwargs) -> None:
+def exactly_one(**kwargs: Any) -> None:
     """
     Verify arguments; exactly one of all keyword arguments must not be None.
 
@@ -313,7 +442,7 @@ def exactly_one(**kwargs) -> None:
 
 
 def spooled_to_bytes_io_if_needed(
-    file_obj: Optional[Union[bytes, BinaryIO, SpooledTemporaryFile]],
+    file_obj: Optional[Union[bytes, BinaryIO, SpooledTemporaryFile[bytes]]],
 ) -> Optional[Union[bytes, BinaryIO]]:
     if isinstance(file_obj, SpooledTemporaryFile):
         file_obj.seek(0)
@@ -344,7 +473,10 @@ def convert_to_bytes(
     return f_bytes
 
 
-def convert_ms_office_table_to_text(table: "docxtable.Table", as_html: bool = True) -> str:
+def convert_ms_office_table_to_text(
+    table: Union["docxtable", "pptxtable"],
+    as_html: bool = True,
+) -> str:
     """
     Convert a table object from a Word document to an HTML table string using the tabulate library.
 
@@ -415,11 +547,15 @@ def document_to_element_list(
     include_page_breaks: bool = False,
     last_modification_date: Optional[str] = None,
     infer_list_items: bool = True,
+    source_format: Optional[str] = None,
+    detection_origin: Optional[str] = None,
+    sort_mode: str = SORT_MODE_XY_CUT,
+    languages: Optional[List[str]] = None,
     **kwargs,
 ) -> List[Element]:
     """Converts a DocumentLayout object to a list of unstructured elements."""
     elements: List[Element] = []
-    sort_mode = kwargs.get("sort_mode", SORT_MODE_XY_CUT)
+
     num_pages = len(document.pages)
     for i, page in enumerate(document.pages):
         page_elements: List[Element] = []
@@ -429,8 +565,9 @@ def document_to_element_list(
         image_width = page_image_metadata.get("width")
         image_height = page_image_metadata.get("height")
 
+        translation_mapping: List[Tuple["LayoutElement", Element]] = []
         for layout_element in page.elements:
-            if image_width and image_height and hasattr(layout_element, "coordinates"):
+            if image_width and image_height and hasattr(layout_element.bbox, "coordinates"):
                 coordinate_system = PixelSpace(width=image_width, height=image_height)
             else:
                 coordinate_system = None
@@ -439,14 +576,15 @@ def document_to_element_list(
                 layout_element,
                 coordinate_system=coordinate_system,
                 infer_list_items=infer_list_items,
+                source_format=source_format if source_format else "html",
             )
-
             if isinstance(element, List):
                 for el in element:
                     if last_modification_date:
                         el.metadata.last_modified = last_modification_date
                     el.metadata.page_number = i + 1
                 page_elements.extend(element)
+                translation_mapping.extend([(layout_element, el) for el in element])
                 continue
             else:
                 if last_modification_date:
@@ -454,21 +592,45 @@ def document_to_element_list(
                 element.metadata.text_as_html = (
                     layout_element.text_as_html if hasattr(layout_element, "text_as_html") else None
                 )
+                try:
+                    if (
+                        isinstance(element, Title) and element.metadata.category_depth is None
+                    ) and any(el.type in ["Headline", "Subheadline"] for el in page.elements):
+                        element.metadata.category_depth = 0
+                except AttributeError:
+                    logger.info("HTML element instance has no attribute type")
+
                 page_elements.append(element)
+                translation_mapping.append((layout_element, element))
             coordinates = (
                 element.metadata.coordinates.points if element.metadata.coordinates else None
             )
+
+            el_image_path = (
+                layout_element.image_path if hasattr(layout_element, "image_path") else None
+            )
+
             _add_element_metadata(
                 element,
                 page_number=i + 1,
                 filetype=image_format,
                 coordinates=coordinates,
                 coordinate_system=coordinate_system,
+                category_depth=element.metadata.category_depth,
+                image_path=el_image_path,
+                detection_origin=detection_origin,
+                languages=languages,
                 **kwargs,
             )
 
+        for layout_element, element in translation_mapping:
+            if hasattr(layout_element, "parent") and layout_element.parent is not None:
+                element_parent = first(
+                    (el for l_el, el in translation_mapping if l_el is layout_element.parent),
+                )
+                element.metadata.parent_id = element_parent.id
         sorted_page_elements = page_elements
-        if sortable and sort_mode == SORT_MODE_XY_CUT:
+        if sortable and sort_mode != SORT_MODE_DONT:
             sorted_page_elements = sort_page_elements(page_elements, sort_mode)
 
         if include_page_breaks and i < num_pages - 1:
@@ -476,3 +638,31 @@ def document_to_element_list(
         elements.extend(sorted_page_elements)
 
     return elements
+
+
+def ocr_data_to_elements(
+    ocr_data: List["LayoutElement"],
+    image_size: Tuple[Union[int, float], Union[int, float]],
+    common_metadata: Optional[ElementMetadata] = None,
+    infer_list_items: bool = True,
+    source_format: Optional[str] = None,
+) -> List[Element]:
+    """Convert OCR layout data into `unstructured` elements with associated metadata."""
+
+    image_width, image_height = image_size
+    coordinate_system = PixelSpace(width=image_width, height=image_height)
+    elements = []
+    for layout_element in ocr_data:
+        element = normalize_layout_element(
+            layout_element,
+            coordinate_system=coordinate_system,
+            infer_list_items=infer_list_items,
+            source_format=source_format if source_format else "html",
+        )
+
+        if common_metadata:
+            element.metadata.update(common_metadata)
+
+        elements.append(element)
+
+    return elements
