@@ -1,3 +1,4 @@
+import copy
 import datetime
 import email
 import os
@@ -19,12 +20,14 @@ from unstructured.partition.common import (
     convert_to_bytes,
     exactly_one,
 )
+from unstructured.partition.lang import apply_lang_metadata
 
 if sys.version_info < (3, 8):
     from typing_extensions import Final
 else:
     from typing import Final
 
+from unstructured.chunking.title import add_chunking_strategy
 from unstructured.cleaners.core import clean_extra_whitespace, replace_mime_encodings
 from unstructured.cleaners.extract import (
     extract_datetimetz,
@@ -55,6 +58,7 @@ from unstructured.partition.html import partition_html
 from unstructured.partition.text import partition_text
 
 VALID_CONTENT_SOURCES: Final[List[str]] = ["text/html", "text/plain"]
+DETECTION_ORIGIN: str = "email"
 
 
 def _parse_received_data(data: str) -> List[Element]:
@@ -127,13 +131,15 @@ def build_email_metadata(
     if sent_to is not None:
         sent_to = [recipient.strip() for recipient in sent_to.split(",")]
 
-    return ElementMetadata(
+    element_metadata = ElementMetadata(
         sent_to=sent_to,
         sent_from=sent_from,
         subject=header_dict.get("Subject"),
         last_modified=metadata_last_modified or email_date,
         filename=filename,
     )
+    element_metadata.detection_origin = DETECTION_ORIGIN
+    return element_metadata
 
 
 def convert_to_iso_8601(time: str) -> Optional[str]:
@@ -210,8 +216,7 @@ def find_embedded_image(
     image_raw_info = element.text[start:end]
     image_info = clean_extra_whitespace(image_raw_info.split(":")[1])
     element.text = element.text.replace("[image: " + image_info[:-1] + "]", "")
-
-    return Image(text=image_info[:-1]), element
+    return Image(text=image_info[:-1], detection_origin="email"), element
 
 
 def parse_email(
@@ -241,6 +246,7 @@ def parse_email(
 
 @process_metadata()
 @add_metadata_with_filetype(FileType.EML)
+@add_chunking_strategy()
 def partition_email(
     filename: Optional[str] = None,
     file: Optional[Union[IO[bytes], SpooledTemporaryFile]] = None,
@@ -255,6 +261,9 @@ def partition_email(
     process_attachments: bool = False,
     attachment_partitioner: Optional[Callable] = None,
     min_partition: Optional[int] = 0,
+    chunking_strategy: Optional[str] = None,
+    languages: Optional[List[str]] = ["auto"],
+    detect_language_per_element: bool = False,
     **kwargs,
 ) -> List[Element]:
     """Partitions an .eml documents into its constituent elements.
@@ -286,6 +295,13 @@ def partition_email(
     min_partition
         The minimum number of characters to include in a partition. Only applies if
         processing the text/plain content.
+    languages
+        User defined value for `metadata.languages` if provided. Otherwise language is detected
+        using naive Bayesian filter via `langdetect`. Multiple languages indicates text could be
+        in either language.
+        Additional Parameters:
+            detect_language_per_element
+                Detect language per element instead of at the document level.
     """
     if content_source not in VALID_CONTENT_SOURCES:
         raise ValueError(
@@ -365,6 +381,8 @@ def partition_email(
             text=content,
             include_metadata=False,
             metadata_filename=metadata_filename,
+            languages=[""],
+            detection_origin="email",
         )
         for element in elements:
             if isinstance(element, Text):
@@ -398,8 +416,10 @@ def partition_email(
             text=content,
             encoding=encoding,
             max_partition=max_partition,
-            metadata_filename=metadata_filename or filename,
             min_partition=min_partition,
+            languages=[""],
+            include_metadata=False,  # metadata is overwritten later, so no need to compute it here
+            detection_origin="email",
         )
 
     for idx, element in enumerate(elements):
@@ -420,7 +440,7 @@ def partition_email(
         metadata_last_modified=metadata_last_modified,
     )
     for element in all_elements:
-        element.metadata = metadata
+        element.metadata = copy.deepcopy(metadata)
 
     if process_attachments:
         with TemporaryDirectory() as tmpdir:
@@ -444,4 +464,12 @@ def partition_email(
                     element.metadata.attached_to_filename = metadata_filename or filename
                     all_elements.append(element)
 
-    return all_elements
+    elements = list(
+        apply_lang_metadata(
+            elements=all_elements,
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+        ),
+    )
+
+    return elements
