@@ -1,7 +1,8 @@
-from typing import IO, TYPE_CHECKING, Dict, List, Optional
+from typing import IO, TYPE_CHECKING, Any, Dict, List, Optional
 
 import requests
 
+from unstructured.chunking.title import add_chunking_strategy
 from unstructured.documents.elements import Element, process_metadata
 from unstructured.documents.html import HTMLDocument
 from unstructured.documents.xml import VALID_PARSERS
@@ -17,6 +18,7 @@ from unstructured.partition.common import (
     get_last_modified_date,
     get_last_modified_date_from_file,
 )
+from unstructured.partition.lang import apply_lang_metadata
 
 if TYPE_CHECKING:
     from unstructured_inference.inference.layout import DocumentLayout
@@ -24,6 +26,7 @@ if TYPE_CHECKING:
 
 @process_metadata()
 @add_metadata_with_filetype(FileType.HTML)
+@add_chunking_strategy()
 def partition_html(
     filename: Optional[str] = None,
     file: Optional[IO[bytes]] = None,
@@ -35,11 +38,16 @@ def partition_html(
     headers: Dict[str, str] = {},
     ssl_verify: bool = True,
     parser: VALID_PARSERS = None,
+    source_format: Optional[str] = None,
     html_assemble_articles: bool = False,
     metadata_filename: Optional[str] = None,
     metadata_last_modified: Optional[str] = None,
     skip_headers_and_footers: bool = False,
-    **kwargs,
+    chunking_strategy: Optional[str] = None,
+    languages: Optional[List[str]] = ["auto"],
+    detect_language_per_element: bool = False,
+    detection_origin: Optional[str] = None,
+    **kwargs: Any,
 ) -> List[Element]:
     """Partitions an HTML document into its constituent elements.
 
@@ -67,17 +75,26 @@ def partition_html(
         in the HTTP request.
     parser
         The parser to use for parsing the HTML document. If None, default parser will be used.
+    source_format
+        The source of the original html. If None we will return HTMLElements but for example
+         partition_rst will pass a value of 'rst' so that we return Title vs HTMLTitle
     metadata_last_modified
         The last modified date for the document.
     skip_headers_and_footers
         If True, ignores any content that is within <header> or <footer> tags
-
+    languages
+        User defined value for `metadata.languages` if provided. Otherwise language is detected
+        using naive Bayesian filter via `langdetect`. Multiple languages indicates text could be
+        in either language.
+        Additional Parameters:
+            detect_language_per_element
+                Detect language per element instead of at the document level.
     """
     if text is not None and text.strip() == "" and not file and not filename and not url:
         return []
+
     # Verify that only one of the arguments was provided
     exactly_one(filename=filename, file=file, text=text, url=url)
-
     last_modification_date = None
     if filename is not None:
         last_modification_date = get_last_modified_date(filename)
@@ -119,12 +136,20 @@ def partition_html(
     if skip_headers_and_footers:
         document = filter_footer_and_header(document)
 
-    return document_to_element_list(
-        document,
-        sortable=False,
-        include_page_breaks=include_page_breaks,
-        last_modification_date=metadata_last_modified or last_modification_date,
-        **kwargs,
+    return list(
+        apply_lang_metadata(
+            document_to_element_list(
+                document,
+                sortable=False,
+                include_page_breaks=include_page_breaks,
+                last_modification_date=metadata_last_modified or last_modification_date,
+                source_format=source_format if source_format else None,
+                detection_origin=detection_origin,
+                **kwargs,
+            ),
+            languages=languages,
+            detect_language_per_element=detect_language_per_element,
+        ),
     )
 
 
@@ -135,6 +160,9 @@ def convert_and_partition_html(
     include_page_breaks: bool = False,
     metadata_filename: Optional[str] = None,
     metadata_last_modified: Optional[str] = None,
+    languages: Optional[List[str]] = ["auto"],
+    detect_language_per_element: bool = False,
+    detection_origin: Optional[str] = None,
 ) -> List[Element]:
     """Converts a document to HTML and then partitions it using partition_html. Works with
     any file format support by pandoc.
@@ -153,6 +181,13 @@ def convert_and_partition_html(
         The filename to use in element metadata.
     metadata_last_modified
         The last modified date for the document.
+    languages
+        User defined value for `metadata.languages` if provided. Otherwise language is detected
+        using naive Bayesian filter via `langdetect`. Multiple languages indicates text could be
+        in either language.
+        Additional Parameters:
+            detect_language_per_element
+                Detect language per element instead of at the document level.
     """
 
     last_modification_date = None
@@ -169,10 +204,14 @@ def convert_and_partition_html(
     # ref: https://github.com/JessicaTegner/pypandoc#usage
     return partition_html(
         text=html_text,
+        source_format=source_format,
         include_page_breaks=include_page_breaks,
         encoding="unicode",
         metadata_filename=metadata_filename,
         metadata_last_modified=metadata_last_modified or last_modification_date,
+        languages=languages,
+        detect_language_per_element=detect_language_per_element,
+        detection_origin=detection_origin,
     )
 
 
