@@ -1,7 +1,8 @@
 import os
+import uuid
 from typing import Any, Dict, List, Optional, Sequence, Union
 
-from unstructured.documents.elements import NoID, Text
+from unstructured.documents.elements import UUID, NoID, Text
 
 VALID_ATTACHMENT_TYPES: List[str] = ["IMAGE", "VIDEO", "RAW_TEXT", "TEXT_URL", "HTML"]
 
@@ -49,7 +50,7 @@ def stage_for_label_box(
     Stages documents to be uploaded to LabelBox and generates LabelBox configuration.
     ref: https://docs.labelbox.com/reference/data-import-format-overview
     """
-    ids: Sequence[Union[str, NoID]]
+    ids: Sequence[Union[str, uuid.UUID, NoID, UUID]]
     if (external_ids is not None) and len(external_ids) != len(elements):
         raise ValueError(
             "The external_ids parameter must be a list and the length of external_ids parameter "
